<?php

namespace Drupal\super_sitemap\Entity;

use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\ContentEntityBase;

/**
 * Defines the SuperSitemapCustomUrl entity.
 *
 * @ContentEntityType(
 *   id = "super_sitemap_custom_url",
 *   label = @Translation("Super Sitemap Custom URL"),
 *   handlers = {
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "action" = "Drupal\Core\Entity\Action\EntityAction",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\Core\Entity\EntityListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "default" = "Drupal\Core\Entity\ContentEntityForm",
 *       "add" = "Drupal\super_sitemap\Form\SuperSitemapCustomUrlForm",
 *       "edit" = "Drupal\super_sitemap\Form\SuperSitemapCustomUrlForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "access" = "Drupal\Core\Entity\EntityAccessControlHandler",
 *   },
 *   base_table = "super_sitemap_custom_url",
 *   admin_permission = "administer super sitemap custom url entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *     "label" = "url",
 *     "owner" = "uid",
 *     "status" = "status"
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/super_sitemap_custom_url/{super_sitemap_custom_url}",
 *     "add-form" = "/admin/structure/super_sitemap_custom_url/add",
 *     "edit-form" = "/admin/structure/super_sitemap_custom_url/{super_sitemap_custom_url}/edit",
 *     "delete-form" = "/admin/structure/super_sitemap_custom_url/{super_sitemap_custom_url}/delete",
 *     "collection" = "/admin/structure/super_sitemap_custom_url",
 *   }
 * )
 */
class SuperSitemapCustomUrl extends ContentEntityBase {
  use EntityChangedTrait;
  use EntityPublishedTrait;

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['url'] = BaseFieldDefinition::create('uri')
      ->setLabel(t('URL'))
      ->setDescription(t('The URL of the custom sitemap entry.'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'uri_link',
    // Adjust the weight to control the order of fields in the form.
        'weight' => 10,
        'settings' => [],
      ])
      ->setDisplayOptions('view', [
    // This configures the label to be hidden in the view.
        'label' => 'hidden',
    // This configures the field to not be displayed in the view mode.
        'type' => 'hidden',
    // The weight, though it's less relevant when field is hidden.
        'weight' => 10,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['url_type'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('URL Type'))
      ->setDescription(t('The type of URL.'))
      ->setSettings([
        'allowed_values' => ['index' => 'Index', 'canonical' => 'Canonical'],
      ])
      ->setDefaultValue('canonical')
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
    // 'options_select' is typically used for list fields in forms.
        'type' => 'options_select',
    // Adjust the weight to control the order of fields in the form.
        'weight' => 10,
        'settings' => [],
      ])
      ->setDisplayOptions('view', [
    // Configures the label to be displayed above the field value in the view.
        'label' => 'above',
    // 'list_default' is typically used for list fields in view modes.
        'type' => 'list_default',
    // Adjust the weight if needed for ordering in view mode.
        'weight' => 10,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['priority'] = BaseFieldDefinition::create('decimal')
      ->setLabel(t('Priority'))
      ->setDescription(t('The priority of the URL in the sitemap.'))
      ->setSettings([
    // Number of decimal places.
        'scale' => 1,
    // Minimum value for the field.
        'min' => '0.1',
    // Maximum value for the field.
        'max' => '1.0',
      ])
      ->setDefaultValue('1.0')
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
    // Use 'number' type for decimal input.
        'type' => 'number',
    // Adjust the weight to control the order of fields in the form.
        'weight' => 10,
        'settings' => [
    // Defines the step for the number field, aligning with the scale.
          'step' => '0.1',
        ],
      ])
      ->setDisplayOptions('view', [
    // Configures the label to be displayed above the field value in the view.
        'label' => 'above',
    // 'number_decimal' is a typical formatter for decimal fields in view mode.
        'type' => 'number_decimal',
    // Adjust the weight if needed for ordering in view mode.
        'weight' => 10,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['change_frequency'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Change frequency'))
      ->setDescription(t('The frequency of the custom sitemap entry.'))
      ->setSettings([
        'allowed_values' => [
          'always' => t('always'),
          'hourly' => t('hourly'),
          'daily' => t('daily'),
          'weekly' => t('weekly'),
          'monthly' => t('monthly'),
          'yearly' => t('yearly'),
          'never' => t('never'),
        ],
      ])
      ->setDefaultValue('monthly')
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 10,
        'settings' => []
      ])
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'list_default',
        'weight' => 10,
        'settings' => []
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setDescription(t('A boolean indicating whether the URL is enabled.'))
      ->setRevisionable(TRUE)
      ->setLabel(t('Status'))
      ->setDefaultValue(TRUE)
      ->setSetting('on_label', 'Enabled')
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => FALSE,
        ],
        'weight' => 10,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'boolean',
        'label' => 'above',
        'weight' => 10,
        'settings' => [
          'format' => 'enabled-disabled',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Author'))
      ->setSetting('target_type', 'user')
      ->setDefaultValueCallback(self::class . '::getDefaultEntityOwner')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => '',
        ],
        'weight' => 15,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'author',
        'weight' => 15,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setTranslatable(TRUE)
      ->setDescription(t('The time that the custom url was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the custom url was last edited.'));

    return $fields;
  }

  /**
   * Returns current user as default entity owner.
   */
  public static function getDefaultEntityOwner() {
    // Get the current user from the service container.
    $current_user = \Drupal::currentUser();
    // Return the current user's ID.
    return ['target_id' => $current_user->id()];
  }

}
