<?php

declare(strict_types=1);

namespace Drupal\super_sitemap\Form;

use Drupal\Component\Plugin\PluginManagerBase;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\super_sitemap\Entity\SuperSitemapDataSource;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The form the Super Sitemap Data Source entity.
 */
final class SuperSitemapDataSourceForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    $form = parent::form($form, $form_state);

    $sitemaps = $this->getConfiguredSitemaps();
    if (empty($sitemaps)) {
      $form['sitemap_title_message'] = [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#value' => $this->t('No sitemaps are configured to use the Super Sitemap XML URL Generator'),
      ];
      $form['sitemap_message'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('Update your <a href="@url">sitemap configuration</a> and then come back to assign your data sources to sitemaps.', [
          '@url' => '/admin/config/search/simplesitemap',
        ]),
      ];
    }
    else {
      $form['label'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Label'),
        '#maxlength' => 255,
        '#default_value' => $this->entity->label(),
        '#required' => TRUE,
      ];

      $form['id'] = [
        '#type' => 'machine_name',
        '#default_value' => $this->entity->id(),
        '#machine_name' => [
          'exists' => [SuperSitemapDataSource::class, 'load'],
        ],
        '#disabled' => !$this->entity->isNew(),
      ];

      $form['status'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enabled'),
        '#default_value' => $this->entity->status(),
      ];

      $form['description'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Description'),
        '#default_value' => $this->entity->get('description'),
      ];

      $form['sitemap'] = [
        '#type' => 'select',
        '#title' => $this->t('Sitemap'),
        '#required' => TRUE,
        '#description' => $this->t('Select which sitemap should be augmented by this data source. <br/>Only sitemaps with a type that supports the "Super Data Source URL generator" are listed here.'),
        '#default_value' => $this->entity->get('sitemap'),
        '#options' => $sitemaps,
      ];

      $form['xml_source'] = [
        '#type' => 'textarea',
        '#title' => $this->t('XML source'),
        '#required' => TRUE,
        '#description' => $this->t('The raw XML representing the external source\'s sitemap.'),
        '#default_value' => $this->entity->get('xml_source'),
      ];
    }

    return $form;
  }

  /**
   * Gather all available data source config into a selectable list of options.
   *
   * @return array
   *   The array of key => label values for data source config.
   */
  public function getConfiguredSitemaps() {
    $configs = \Drupal::entityTypeManager()->getStorage('simple_sitemap')->loadMultiple();
    $options = [];
    foreach ($configs as $id => $config) {
      $url_generators = $config->getType()->getUrlGenerators();
      if (array_key_exists('super_xml_data_source_url', $url_generators)) {
        $options[$id] = $config->label();
      }
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);

    // Validate that the value in the 'XML source' field is valid sitemap XML.
    $xml_source = $form_state->getValue('xml_source');
    if (!empty($xml_source)) {
      if (!$this->isValidSitemapXml($xml_source)) {
        $form_state->setErrorByName('xml_source', $this->t('The provided XML is not a valid sitemap XML. It must contain at least one <url> element with a <loc> element.'));
      }
    }
  }

  /**
   * Validates if the XML source is a valid sitemap XML.
   *
   * @param string $xml_source
   *   The XML source to validate.
   *
   * @return bool
   *   TRUE if the XML is valid, FALSE otherwise.
   */
  private function isValidSitemapXml(string $xml_source): bool {
    if (empty($xml_source)) {
      return FALSE;
    }

    try {
      // Load and validate the XML by checking that there is at least 1 <url>
      // element with a <loc> value.
      $dom = new \DOMDocument();
      $dom->loadXML($xml_source, LIBXML_NOWARNING | LIBXML_NOERROR);
      $urls = $dom->getElementsByTagName('url');
      if ($urls->length === 0) {
        return FALSE;
      }
      foreach ($urls as $url) {
        $loc = $url->getElementsByTagName('loc');
        if ($loc->length === 0) {
          return FALSE;
        }
      }

      return TRUE;
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new example %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated example %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
