<?php

namespace Drupal\super_sitemap\Plugin\simple_sitemap\UrlGenerator;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Url;
use Drupal\simple_sitemap\Exception\SkipElementException;
use Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\UrlGeneratorBase;
use Drupal\simple_sitemap\Logger;
use Drupal\simple_sitemap\Settings;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\simple_sitemap\Entity\EntityHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\simple_sitemap\Plugin\simple_sitemap\SimpleSitemapPluginBase;

/**
 * Provides a Super Sitemap Custom URL generator.
 *
 * @UrlGenerator(
 * id = "super_sitemap_custom_canonical_url",
 * label = @Translation("Super sitemap Custom canonical URL"),
 * description = @Translation("Generates custom URLs with a canonical URL type."),
 * )
 */
class SuperSitemapCustomEntityUrlGenerator extends UrlGeneratorBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * ViewsUrlGenerator constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\simple_sitemap\Logger $logger
   *   The simple_sitemap.logger service.
   * @param \Drupal\simple_sitemap\Settings $settings
   *   The simple_sitemap.settings service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\simple_sitemap\Entity\EntityHelper $entity_helper
   *   The simple_sitemap.entity_helper service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    Logger $logger,
    Settings $settings,
    LanguageManagerInterface $language_manager,
    EntityTypeManagerInterface $entity_type_manager,
    EntityHelper $entity_helper,
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $logger,
      $settings,
      $language_manager,
      $entity_type_manager,
      $entity_helper
    );
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): SimpleSitemapPluginBase {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('simple_sitemap.logger'),
      $container->get('simple_sitemap.settings'),
      $container->get('language_manager'),
      $container->get('entity_type.manager'),
      $container->get('simple_sitemap.entity_helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataSets(): array {
    return $this->entityTypeManager
      ->getStorage('super_sitemap_custom_url')
      ->getQuery()
      ->condition('status', 1)
      ->condition('url_type', 'canonical')
      ->accessCheck(FALSE)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  protected function processDataSet($data_set): array {
    $entity = $this->entityTypeManager->getStorage('super_sitemap_custom_url')->load($data_set);
    if ($entity) {
      $date = DrupalDateTime::createFromTimestamp($entity->changed->value);
      $settings = [
        'lastmod' => $date->format('Y-m-d\TH:i:sP'),
        'changefreq' => $entity->change_frequency->value,
        'priority' => $entity->priority->value,
      ];
      $url = Url::fromUri($entity->url->value);
      $path_data = $this->constructPathData($url, $settings);
      return $path_data;
    }
    throw new SkipElementException();
  }

}
