<?php

namespace Drupal\super_sitemap\Plugin\simple_sitemap\UrlGenerator;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\simple_sitemap\Logger;
use Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\UrlGeneratorBase;
use Drupal\simple_sitemap\Settings;
use Drupal\super_sitemap\Event\SuperSitemapDataSourceRequestEvent;
use Drupal\super_sitemap\Event\SuperSitemapDataSourceResponseEvent;
use Drupal\super_sitemap\Event\SuperSitemapEvents;
use Drupal\super_sitemap\SuperSitemapDataSourceInterface;
use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the generator for the XML data source.
 *
 * @UrlGenerator(
 *   id = "super_xml_data_source_url",
 *   label = @Translation("Super XML Data Source URL generator"),
 * )
 */
class SuperXmlDataSourceUrlGenerator extends UrlGeneratorBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * SuperXmlDataSourceUrlGenerator constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\simple_sitemap\Logger $logger
   *   Simple XML Sitemap logger.
   * @param \Drupal\simple_sitemap\Settings $settings
   *   The simple_sitemap.settings service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    Logger $logger,
    Settings $settings,
    EntityTypeManagerInterface $entity_type_manager,
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $logger,
      $settings
    );
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition): SuperXmlDataSourceUrlGenerator {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('simple_sitemap.logger'),
      $container->get('simple_sitemap.settings'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataSets(): array {
    $sitemap = $this->sitemap;
    $data_sources = $this->entityTypeManager
      ->getStorage('super_sitemap_data_source')
      ->loadByProperties([
        'sitemap' => $sitemap->id(),
        'status' => 1,
      ]);

    // Allow the request to fail gracefully if there aren't any data sources
    // for some reason. This can happen if a data source is removed but the
    // configuration is not updated to reflect the change or all of the data
    // sources have been disabled.
    if (empty($data_sources)) {
      \Drupal::logger('super_sitemap')->debug('Data source is empty!');
      return [];
    }

    // Gather all the data sources that have the same sitemap set and store the
    // links across all those data sources in a single collection.
    $combined_data = [];
    foreach ($data_sources as $data_source) {
      $source_data = $this->getDataFromSource($data_source);
      if (!empty($source_data) && is_array($source_data)) {
        $combined_data = array_merge($combined_data, $source_data);
      }
    }

    return $combined_data;
  }

  /**
   * Fetch data from the configured data source.
   *
   * @return array
   *   The array of results from the data source.
   */
  protected function getDataFromSource(SuperSitemapDataSourceInterface $data_source) {
    $results = [];

    // Extract the values from each URL entry in the data source and store them
    // in an array for processing and inclusion into the sitemap.
    $xml_input = $data_source->getXmlSource() ?? '';
    if (!empty($xml_input)) {
      try {
        $xml = simplexml_load_string($xml_input);
        if ($xml && isset($xml->url)) {
          foreach ($xml->url as $url) {
            $item = [];
            $loc = (string) $url->loc;
            if (!empty($loc)) {
              $item['loc'] = $loc;
            }

            $lastmod = (string) $url->lastmod;
            if (!empty($lastmod)) {
              $item['lastmod'] = $lastmod;
            }

            $priority = $url->priority;
            if (!empty($priority)) {
              $item['priority'] = $priority;
            }

            $results[] = $item;
          }
        }
      }
      catch (\Exception $e) {
        $this->logger->log('Failed to parse XML from input for @data_source.', [
          '@data_source' => $data_source->id(),
        ]);
      }
    }
    return $results;
  }

  /**
   * {@inheritdoc}
   */
  protected function processDataSet($data_set): array {
    $path_data = [
      'url' => $data_set['loc'],
      'lastmod' => $data_set['lastmod'] ?? NULL,
      'priority' => $data_set['priority'] ?? NULL,
      'changefreq' => !empty($data_set['changefreq']) ? $data_set['changefreq'] : NULL,
    ];

    return $path_data;
  }

}
