<?php

namespace Drupal\super_sitemap\Queue;

use Drupal\simple_sitemap\Queue\QueueWorker;
use Drupal\Component\Utility\Timer;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\Lock\LockBackendInterface;
use Drupal\simple_sitemap\Entity\SimpleSitemap;
use Drupal\simple_sitemap\Logger;
use Drupal\simple_sitemap\Settings;
use Drupal\simple_sitemap\Queue\SimpleSitemapQueue;

/**
 * Class QueueWorkerOverrider.
 *
 * Overrides the decorated class's generate method.
 */
class QueueWorkerOverrider extends QueueWorker {

    /**
     * The decorated service.
     *
     * @var \Drupal\simple_sitemap\Queue\QueueWorker
     */
    protected QueueWorker $queueWorker;

    /**
     * QueueWorkerOverride constructor.
     *
     * @param \Drupal\simple_sitemap\Queue\QueueWorker $queue_worker
     *  The original decorated queue worker service.
     * @param \Drupal\simple_sitemap\Settings $settings
     *  The simple_sitemap.settings service.
     * @param \Drupal\Core\KeyValueStore\KeyValueFactoryInterface $key_value
     *  The Key/Value factory service.
     * @param \Drupal\simple_sitemap\Queue\SimpleSitemapQueue $element_queue
     *  Simple XML Sitemap queue handler.
     * @param \Drupal\simple_sitemap\Logger $logger
     *  Simple XML Sitemap logger.
     * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
     *  The module handler service.
     * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
     *  The entity type manager.
     * @param \Drupal\Core\Lock\LockBackendInterface $lock
     *  The lock backend that should be used.
     *
     */
    public function __construct(
        QueueWorker $queue_worker,
        Settings $settings,
        KeyValueFactoryInterface $key_value,
        SimpleSitemapQueue $element_queue,
        Logger $logger,
        ModuleHandlerInterface $module_handler,
        EntityTypeManagerInterface $entity_type_manager,
        LockBackendInterface $lock,
    ) {
        $this->queueWorker = $queue_worker;
        parent::__construct(
            $settings,
            $key_value,
            $element_queue,
            $logger,
            $module_handler,
            $entity_type_manager,
            $lock
        );
    }

    /**
     * Generates all sitemaps.
     *
     * The majority of this logic is directly from the decorated class
     * \Drupal\simple_sitemap\Queue\QueueWorker. This overrides the decorated
     * service with the $this->getSitemapTypeMaxLinkConfig usage.
     *
     * @param string $from
     *   The source of generation.
     *
     * @return $this
     *
     * @throws \Drupal\Component\Plugin\Exception\PluginException
     */
    public function generate(string $from = self::GENERATE_TYPE_FORM): QueueWorker {

        $this->generatorSettings = [
            'base_url' => $this->settings->get('base_url', ''),
            'xsl' => $this->settings->get('xsl', TRUE),
            'default_variant' => $this->settings->get('default_variant'),
            'skip_untranslated' => $this->settings->get('skip_untranslated', FALSE),
            'remove_duplicates' => $this->settings->get('remove_duplicates', TRUE),
            'excluded_languages' => $this->settings->get('excluded_languages', []),
        ];
        $this->maxLinks = $this->settings->get('max_links', 2000);
        $max_execution_time = $this->settings->get('generate_duration', 10000);
        Timer::start('simple_sitemap_generator');

        $this->unstashResults();

        if (!$this->generationInProgress()) {
            $this->rebuildQueue(SimpleSitemap::loadMultiple());
        }

        // Acquire a lock for max execution time + 5 seconds. If max_execution time
        // is unlimited then lock for 1 hour.
        $lock_timeout = $max_execution_time > 0 ? ($max_execution_time / 1000) + 5 : static::GENERATE_LOCK_TIMEOUT;
        if (!$this->lock->acquire(static::LOCK_ID, $lock_timeout)) {
            throw new \RuntimeException('Unable to acquire a lock for sitemap generation');
        }

        foreach ($this->queue->yieldItem() as $element) {

            if (!empty($max_execution_time) && Timer::read('simple_sitemap_generator') >= $max_execution_time) {
                break;
            }

            try {
                if ($this->sitemapProcessedNow && !$this->sitemapProcessedNow->isEnabled()) {
                    $this->queue->deleteItem($element);
                    $this->elementsRemaining--;
                    continue;
                }

                if ($this->sitemapProcessedNow === NULL || $element->data['sitemap'] !== $this->sitemapProcessedNow->id()) {

                    if (NULL !== $this->sitemapProcessedNow) {
                        $this->generateSitemapChunksFromResults(TRUE);
                        $this->publishCurrentSitemap();
                    }

                    $this->processedPaths = [];
                    if (($this->sitemapProcessedNow = $this->entityTypeManager->getStorage('simple_sitemap')->load($element->data['sitemap'])) === NULL) {
                        $this->queue->deleteItem($element);
                        $this->elementsRemaining--;
                        continue;
                    }
                }

                $this->generateResultsFromElement($element);

                // Use the sitemap's type max links configuration if available.
                $this->getSitemapTypeMaxLinkConfig($element->data['sitemap']);

                if (!empty($this->maxLinks) && count($this->results) >= $this->maxLinks) {
                    $this->generateSitemapChunksFromResults();
                }
            }
            catch (\Exception $e) {
                $this->logger->logException($e);
            }

            $this->queue->deleteItem($element);
            $this->elementsRemaining--;
        }

        if ($this->getQueuedElementCount() === 0) {
            $this->generateSitemapChunksFromResults(TRUE);
            $this->publishCurrentSitemap();
        }
        else {
            $this->stashResults();
        }
        $this->lock->release(static::LOCK_ID);

        return $this;
    }

    /**
     * Set max links using the sitemap's type configured setting.
     *
     * @param string $sitemap
     *   The sitemap variant machine name.
     *
     * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
     * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
     */
    protected function getSitemapTypeMaxLinkConfig($sitemap): void {
        /** @var \Drupal\simple_sitemap\Entity\SimpleSitemap $simple_sitemap */
        $simple_sitemap = $this->entityTypeManager->getStorage('simple_sitemap')->load($sitemap);
        $sitemap_type = $simple_sitemap->getType();

        $max_links = $sitemap_type->getThirdPartySetting('super_sitemap', 'max_links', NULL);
        if ($max_links) {
            $this->maxLinks = $max_links;
        }
    }

}
