<?php

namespace Drupal\super_sitemap\Service;

use Drupal\ultimate_cron\Entity\CronJob;
use Drupal\simple_sitemap\Entity\SimpleSitemap;
use Drupal\simple_sitemap\Queue\QueueWorker;
use Drupal\simple_sitemap\Manager\Generator;

/**
 * Provides services for the Super Sitemap module.
 */
class SuperSitemap {

  /**
   * The queue worker service.
   *
   * @var \Drupal\simple_sitemap\Queue\QueueWorker
   */
  protected $queueWorker;

  /**
   * The generator service.
   *
   * @var \Drupal\simple_sitemap\Manager\Generator
   */
  protected $generator;

  /**
   * Constructs a SuperSitemap service.
   *
   * @param \Drupal\simple_sitemap\Queue\QueueWorker $queue_worker
   *   The simple_sitemap.queue_worker service.
   * @param \Drupal\simple_sitemap\Manager\Generator $generator
   *   The simple_sitemap.generator service.
   */
  public function __construct(QueueWorker $queue_worker, Generator $generator) {
    $this->queueWorker = $queue_worker;
    $this->generator = $generator;
  }

  /**
   * Creates cron jobs for sitemap variants.
   */
  public function createCronJobs(): void {
    $sitemap_variants = SimpleSitemap::loadMultiple();

    foreach ($sitemap_variants as $sitemap_variant) {
      $this->createCronJob($sitemap_variant);
    }
  }

  /**
   * Creates a cron job for a specific sitemap variant.
   *
   * @param \Drupal\simple_sitemap\Entity\SimpleSitemap $sitemap_variant
   *   The sitemap variant entity.
   */
  public function createCronJob(SimpleSitemap $sitemap_variant): void {
    $job = CronJob::load($sitemap_variant->id());
    if (!$job) {
      $values = [
        'title' => $sitemap_variant->label(),
        'id' => $sitemap_variant->id(),
        'module' => 'super_sitemap',
        'callback' => 'super_sitemap.common:superSitemapVariantCronJob',
      ];
      $job = CronJob::create($values);
      $job->save();
    }
  }

  /**
   * Executes the cron job for a specific sitemap variant.
   *
   * @param \Drupal\ultimate_cron\Entity\CronJob $job
   *   The cron job entity.
   */
  public function superSitemapVariantCronJob(CronJob $job): void {
    $variant_name = $job->id();

    // Load the sitemap variant.
    $sitemap_variant = SimpleSitemap::loadMultiple([$variant_name]);

    // Rebuild the queue for the sitemap variant.
    $this->queueWorker->rebuildQueue($sitemap_variant);

    // Generate the sitemap.
    $this->generator->generate('cron');
  }

}
